@@ -1,5 +1,7 @@
+from __future__ import annotations
+
 import datetime
-from typing import Generic, Literal, Optional, TypeVar, Union
+from typing import Any, Generic, Literal, Optional, TYPE_CHECKING, TypeVar, Union
 import warnings
 
 import phonenumbers
@@ -9,6 +11,9 @@ from pydantic import generics
 # Must use typing.Dict etc for generics not native as of pydantic 1.7.3
 from typing import List, Dict  # isort: skip
 
+if TYPE_CHECKING:
+    from collections.abc import Iterator
+
 DataT = TypeVar("DataT")
 
 TYPES_SEX = Literal["Male", "Female"]  # also 'Unknown'
@@ -142,39 +147,49 @@ TYPES_PERMIT_CATEGORIES = Literal[
     "Without Compound Bows",
 ]
 TYPES_PERMIT_TYPE = Literal["Leadership", "Supervisory"]
+TYPES_DISCLOSURE_PROVIDERS = Literal["Atlantic Data", "DBS Paper Application", "Other"]
+TYPES_DISCLOSURE_STATUSES = Literal[
+    "Application Withdrawn",
+    "Application submitted - in progress",
+    "Disclosure Expired",
+    "Disclosure Issued",
+    "Expired",
+    "ID check required",
+    "ID selection required",
+]
 
 
 class MemberGenericDict(generics.GenericModel, Generic[DataT]):
     __root__: Dict[int, DataT]  # Must use typing.Dict not dict as of pydantic 1.7.3
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator[tuple[int, DataT]]:
         """Iterate over model items."""
         yield from self.__root__.items()
 
-    def __getitem__(self, item):
+    def __getitem__(self, item: int) -> DataT:
         """Get item by key."""
         return self.__root__[item]
 
-    def __len__(self):
+    def __len__(self) -> int:
         """Get number of items."""
         return len(self.__root__)
 
-    def items(self):
+    def items(self) -> Iterator[tuple[int, DataT]]:
         yield from iter(self)
 
 
 class MemberGenericList(generics.GenericModel, Generic[DataT]):
     __root__: List[DataT]
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator[DataT]:
         """Iterate over model items."""
-        return iter(self.__root__)
+        yield from self.__root__
 
-    def __getitem__(self, item):
+    def __getitem__(self, item: int) -> DataT:
         """Get item by index."""
         return self.__root__[item]
 
-    def __len__(self):
+    def __len__(self) -> int:
         """Get number of items."""
         return len(self.__root__)
 
@@ -219,7 +234,7 @@ class MemberDetails(MemberBase):
     # TODO - potential disabilities, qualifications, hobbies sections
 
     @pydantic.validator("main_phone")
-    def check_phone_number(cls, v, values):
+    def check_phone_number(cls, v: Optional[str], values: dict[str, Any]) -> Optional[str]:
         if v is None or not v or v == "0":
             return None
 
@@ -234,7 +249,7 @@ class MemberDetails(MemberBase):
             warnings.warn(f"Member No {cn}: phone number {v} is not valid!", RuntimeWarning)
 
         fmt = phonenumbers.PhoneNumberFormat.NATIONAL if n.country_code == 44 else phonenumbers.PhoneNumberFormat.INTERNATIONAL
-        return phonenumbers.format_number(n, fmt)
+        return str(phonenumbers.format_number(n, fmt))
 
 
 # Roles Tab (Main List - item)
@@ -384,3 +399,14 @@ class MemberPermit(MemberBase):
 
 # Permits Tab - collection
 MemberPermitsList = MemberGenericList[MemberPermit]
+
+
+class MemberDisclosure(pydantic.BaseModel):
+    country: Optional[Literal["England & Wales", "The Scout Association"]]  # TODO Scot, Wales, NI, BSO, Branches, Channel Islands
+    provider: TYPES_DISCLOSURE_PROVIDERS
+    type: Literal["Enhanced with Barring"]
+    number: Optional[int]  # If Application Withdrawn, no disclosure number
+    issuer: Optional[TYPES_DISCLOSURE_PROVIDERS]
+    issue_date: Optional[datetime.date]  # If Application Withdrawn, maybe no issue date
+    status: TYPES_DISCLOSURE_STATUSES
+    expiry_date: Optional[datetime.date]  # If Application Withdrawn, no expiry date
